@@ -15,6 +15,7 @@
 #include "gnome-theme-details.h"
 #include "gnome-theme-installer.h"
 #include "gnome-theme-info.h"
+#include "gtkrc-utils.h"
 
 #define MAX_ELEMENTS_BEFORE_SCROLLING 12
 
@@ -46,7 +47,9 @@ static void load_theme_names                (GtkTreeView        *tree_view,
 					     GList              *theme_list,
 					     const gchar        *default_theme);
 static char *path_to_theme_id               (const char *path);
+static void update_color_buttons_from_string (gchar *color_scheme);
 
+void toggle_color_scheme_key (GtkWidget *checkbutton, gpointer *data);
 
 static char *
 path_to_theme_id (const char *path)
@@ -123,6 +126,7 @@ setup_tree_view (GtkTreeView *tree_view,
  					       renderer,
  					       "text", THEME_NAME_COLUMN,
  					       NULL);
+  g_object_set (renderer, "ellipsize", PANGO_ELLIPSIZE_END, NULL);
 }
 
 static void
@@ -133,8 +137,8 @@ gtk_theme_update_remove_button (GtkTreeSelection *selection,
   gchar *theme_selected;
   GtkTreeModel *model;
   GtkTreeIter iter;
-  GList *theme_list=NULL, *string_list, *list;
-  gchar *theme_dir = NULL;
+  GList *theme_list=NULL, *list;
+  gchar *theme_base=NULL;
   GnomeVFSResult vfs_result;
   GnomeVFSFileInfo *vfs_info;
 
@@ -148,12 +152,12 @@ gtk_theme_update_remove_button (GtkTreeSelection *selection,
   if (theme_selected != NULL) 
   {
      switch (theme_type) {
-		case THEME_GTK: theme_dir = g_strdup("/gtk-2.0/");
+		case THEME_GTK: theme_base = g_strdup("/gtk-2.0/");
 				theme_list = gnome_theme_info_find_by_type (GNOME_THEME_GTK_2);
 				break;
 		case THEME_ICON: theme_list = gnome_theme_icon_info_find_all();
 				 break;
-		case THEME_WINDOW: theme_dir = g_strdup("/metacity-1/");
+		case THEME_WINDOW: theme_base = g_strdup("/metacity-1/");
 				    theme_list = gnome_theme_info_find_by_type (GNOME_THEME_METACITY);
 				    break;
 		default: theme_list = NULL;
@@ -161,22 +165,25 @@ gtk_theme_update_remove_button (GtkTreeSelection *selection,
     }
   }
 		
-  string_list = NULL;
+  vfs_info = gnome_vfs_file_info_new ();
 
   for (list = theme_list; list; list = list->next)
   {
     GnomeThemeInfo *info = list->data;
+	gchar *theme_dir = NULL;
+
     if (!strcmp(info->name, theme_selected))
     {
 	if (theme_type == THEME_ICON) 
 		theme_dir = g_strdup(info->path);
 	else 
-		theme_dir = g_strdup_printf("%s/%s", info->path, theme_dir);
+		theme_dir = g_strdup_printf("%s/%s", info->path, theme_base);
 			
-	vfs_info = gnome_vfs_file_info_new ();
 	vfs_result = gnome_vfs_get_file_info (theme_dir,
 					      vfs_info,
 				              GNOME_VFS_FILE_INFO_GET_ACCESS_RIGHTS);
+	g_free (theme_dir);
+
         if (vfs_result == GNOME_VFS_OK) 
 	{
 		if ((vfs_info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_ACCESS) &&
@@ -187,8 +194,58 @@ gtk_theme_update_remove_button (GtkTreeSelection *selection,
 	} else {
 		gtk_widget_set_sensitive(remove_button, FALSE);
 	}
+
       }
     }
+    gnome_vfs_file_info_unref(vfs_info);
+    g_free (theme_base);
+    g_free (theme_selected);
+}
+
+static void
+update_color_scheme_tab ()
+{
+	GSList *symbolic_colors = NULL;
+	GSList *engines = NULL;
+	gboolean fg, bg, base, text, fg_s, bg_s, enable_colors;
+	gchar *filename, *theme_name;
+	GtkSettings *settings;
+	GladeXML *dialog;
+
+	dialog = gnome_theme_manager_get_theme_dialog ();
+
+	settings = gtk_settings_get_default ();
+	g_object_get (G_OBJECT (settings), "gtk-theme-name", &theme_name, NULL);
+	filename = gtkrc_find_named (theme_name);
+
+	gtkrc_get_details (filename, &engines, &symbolic_colors);
+	fg = (g_slist_find_custom (symbolic_colors, "fg_color", g_str_equal) != NULL);
+	bg = (g_slist_find_custom (symbolic_colors, "bg_color", g_str_equal) != NULL);
+	base = (g_slist_find_custom (symbolic_colors, "base_color", g_str_equal) != NULL);
+	text = (g_slist_find_custom (symbolic_colors, "text_color", g_str_equal) != NULL);
+	fg_s = (g_slist_find_custom (symbolic_colors, "selected_fg_color", g_str_equal) != NULL);
+	bg_s = (g_slist_find_custom (symbolic_colors, "selected_bg_color", g_str_equal) != NULL);
+
+	enable_colors = (fg && bg && base && text && fg_s && bg_s);
+	gtk_widget_set_sensitive (WID ("enable_custom_colors_checkbutton"), enable_colors);
+
+	if (enable_colors)
+	{
+		gboolean enable_color_schemes;
+		enable_color_schemes = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (WID ("enable_custom_colors_checkbutton")));
+		gtk_widget_set_sensitive (WID ("color_scheme_table"), enable_color_schemes);
+		gtk_widget_set_sensitive (WID ("saved_color_schemes_hbox"), enable_color_schemes);
+		gtk_widget_hide (WID ("color_scheme_message_hbox"));
+	}
+	else
+	{
+		gtk_widget_show (WID ("color_scheme_message_hbox"));
+		gtk_widget_set_sensitive (WID ("color_scheme_table"), FALSE);
+		gtk_widget_set_sensitive (WID ("saved_color_schemes_hbox"), FALSE);
+	}
+
+	g_free (filename);
+	g_free (theme_name);
 }
 
 static void
@@ -196,9 +253,9 @@ gtk_theme_selection_changed (GtkTreeSelection *selection,
 			     gpointer          data)
 {
   GladeXML *dialog;
-		
+
   dialog = gnome_theme_manager_get_theme_dialog ();
-  
+
   update_gconf_key_from_selection (selection, GTK_THEME_KEY);
   gtk_theme_update_remove_button(selection, WID("control_remove_button"), THEME_GTK);
 }
@@ -432,7 +489,7 @@ remove_theme(GtkWidget *button, gpointer data)
 								GTK_BUTTONS_OK_CANCEL,
 								_("Theme deleted succesfully. Please select another theme."));
 					gtk_list_store_remove (GTK_LIST_STORE(model), &iter);
-					gtk_dialog_run (info_dialog);
+					gtk_dialog_run (GTK_DIALOG (info_dialog));
 
 				} else {
 					info_dialog = gtk_message_dialog_new (NULL,
@@ -440,7 +497,7 @@ remove_theme(GtkWidget *button, gpointer data)
 								GTK_MESSAGE_ERROR,
 								GTK_BUTTONS_OK,
 								_("Theme can not be deleted"));
-					gtk_dialog_run (info_dialog);
+					gtk_dialog_run (GTK_DIALOG (info_dialog));
 				}
 				gtk_widget_destroy (info_dialog);
 				gnome_vfs_uri_list_free (uri_list);
@@ -450,11 +507,164 @@ remove_theme(GtkWidget *button, gpointer data)
   }
 }
 
+void
+color_select (GtkWidget *colorbutton, GladeXML *dialog)
+{
+  GConfClient *client = NULL;
+  gchar *new_scheme;
+  GdkColor colors[6];
+  gchar *bg, *fg, *text, *base, *selected_fg, *selected_bg;
+  GtkWidget *widget;
+  GtkTreeModel *model;
+  GtkTreeIter tmp, iter;
+
+  widget = WID ("fg_colorbutton");
+  gtk_color_button_get_color (GTK_COLOR_BUTTON (widget), &colors[0]);
+  widget = WID ("bg_colorbutton");
+  gtk_color_button_get_color (GTK_COLOR_BUTTON (widget), &colors[1]);
+  widget = WID ("text_colorbutton");
+  gtk_color_button_get_color (GTK_COLOR_BUTTON (widget), &colors[2]);
+  widget = WID ("base_colorbutton");
+  gtk_color_button_get_color (GTK_COLOR_BUTTON (widget), &colors[3]);
+  widget = WID ("selected_fg_colorbutton");
+  gtk_color_button_get_color (GTK_COLOR_BUTTON (widget), &colors[4]);
+  widget = WID ("selected_bg_colorbutton");
+  gtk_color_button_get_color (GTK_COLOR_BUTTON (widget), &colors[5]);
+
+
+  fg = g_strdup_printf ("fg_color:#%.4x%.4x%.4x\n", colors[0].red, colors[0].green, colors[0].blue);
+  bg = g_strdup_printf ("bg_color:#%.4x%.4x%.4x\n", colors[1].red, colors[1].green, colors[1].blue);
+  text = g_strdup_printf ("text_color:#%.4x%.4x%.4x\n", colors[2].red, colors[2].green, colors[2].blue);
+  base = g_strdup_printf ("base_color:#%.4x%.4x%.4x\n", colors[3].red, colors[3].green, colors[3].blue);
+  selected_fg = g_strdup_printf ("selected_fg_color:#%.4x%.4x%.4x\n", colors[4].red, colors[4].green, colors[4].blue);
+  selected_bg = g_strdup_printf ("selected_bg_color:#%.4x%.4x%.4x", colors[5].red, colors[5].green, colors[5].blue);
+
+  new_scheme = g_strconcat (fg, bg, text, base, selected_fg, selected_bg, NULL);
+
+  client = gconf_client_get_default ();
+
+  /* Currently we assume this has only been called when one of the colours has
+   * actually changed, so we don't check the original key first
+   */
+  gconf_client_set_string (client, COLOR_SCHEME_KEY, new_scheme, NULL);
+
+  g_object_unref (G_OBJECT (client));
+  free_all (fg, bg, text, base, selected_fg, selected_bg, new_scheme, NULL);
+
+  model = gtk_combo_box_get_model (GTK_COMBO_BOX (WID ("color_scheme_combobox")));
+  for (gtk_tree_model_get_iter_first(GTK_TREE_MODEL (model), &tmp);gtk_tree_model_iter_next(model, &tmp);)
+  {
+    iter = tmp;
+  }
+  gtk_combo_box_set_active_iter (GTK_COMBO_BOX (WID ("color_scheme_combobox")), &iter);
+
+}
+
+void
+toggle_color_scheme_key (GtkWidget *checkbutton, gpointer *data)
+{
+  GConfClient *client = NULL;
+  gboolean use_custom_colors;
+  GladeXML *dialog;
+  GtkSettings *settings;
+  gchar *color_scheme = NULL;
+
+  dialog = gnome_theme_manager_get_theme_dialog ();
+  use_custom_colors = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbutton));
+
+  gtk_widget_set_sensitive (WID ("color_scheme_table"), use_custom_colors);
+  gtk_widget_set_sensitive (WID ("saved_color_schemes_hbox"), use_custom_colors);
+
+  if (!use_custom_colors)
+  {
+    client = gconf_client_get_default ();
+    gconf_client_set_string (client, COLOR_SCHEME_KEY, "", NULL);
+    g_object_unref (G_OBJECT (client));
+  }
+  else
+  {
+    settings = gtk_settings_get_default ();
+    g_object_get (G_OBJECT (settings), "gtk-color-scheme", &color_scheme, NULL);
+    update_color_buttons_from_string (color_scheme);
+    g_free (color_scheme);
+  }
+}
+
+void
+theme_notebook_changed_page (GtkWidget *widget, GladeXML *dialog)
+{
+	update_color_scheme_tab ();
+}
+
+void
+save_color_scheme (GtkWidget *widget, GtkWidget *parent)
+{
+	GtkWidget *dialog, *hbox, *label, *entry;
+	dialog = gtk_dialog_new_with_buttons (_("Save Color Scheme"),
+						GTK_WINDOW (parent),
+						GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_NO_SEPARATOR,
+						GTK_STOCK_CANCEL,
+						GTK_RESPONSE_CANCEL,
+						GTK_STOCK_SAVE,
+						GTK_RESPONSE_ACCEPT, NULL);
+
+	hbox = gtk_hbox_new (FALSE, 6);
+	label = gtk_label_new (_("Save color scheme as:"));
+	entry = gtk_entry_new ();
+
+	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 6);
+	gtk_box_pack_start (GTK_BOX (hbox), entry, FALSE, TRUE, 6);
+
+	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), hbox, TRUE, TRUE, 12);
+	gtk_widget_show_all (hbox);
+
+	gtk_dialog_run (GTK_DIALOG (dialog));
+	gtk_widget_hide (dialog);
+}
+
+void
+delete_color_scheme (GtkWidget *widget, GtkWidget *parent)
+{
+	GladeXML *dialog;
+	gchar *color_scheme_name;
+	GtkWidget *query;
+	dialog = gnome_theme_manager_get_theme_dialog ();
+
+	color_scheme_name = gtk_combo_box_get_active_text (GTK_COMBO_BOX (WID ("color_scheme_combobox")));
+	query = gtk_message_dialog_new (GTK_WINDOW (parent),
+				GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+				GTK_MESSAGE_QUESTION,
+				GTK_BUTTONS_NONE,
+				_("Are you sure you want to delete the \"%s\" color scheme?"),
+				color_scheme_name);
+	gtk_dialog_add_buttons (GTK_DIALOG (query), GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+					GTK_STOCK_DELETE, GTK_RESPONSE_OK, NULL);
+	gtk_dialog_run (GTK_DIALOG (query));
+	gtk_widget_hide (query);
+}
+
+void
+color_scheme_combobox_changed (GtkWidget *widget, GtkWidget *parent)
+{
+	GladeXML *dialog;
+	dialog = gnome_theme_manager_get_theme_dialog ();
+	gchar *color_scheme_name;
+	gboolean custom;
+
+	color_scheme_name = gtk_combo_box_get_active_text (GTK_COMBO_BOX (WID ("color_scheme_combobox")));
+
+	custom = (color_scheme_name != NULL && strcmp (color_scheme_name, _("Custom")) == 0);
+	gtk_widget_set_sensitive (WID ("save_color_scheme_button"), custom);
+	gtk_widget_set_sensitive (WID ("delete_color_scheme_button"), !custom);
+
+}
+
 void
 gnome_theme_details_init (void)
 {
   GtkWidget *parent, *widget;
   GladeXML *dialog;
+  gchar *color_scheme;
 
   if (theme_details_initted)
     return;
@@ -491,7 +701,38 @@ gnome_theme_details_init (void)
   widget = WID ("icon_install_button");
   g_signal_connect (G_OBJECT (widget), "clicked", G_CALLBACK (gnome_theme_installer_run_cb), parent);
 
+  /* color preferences */
+  widget = WID ("fg_colorbutton");
+  g_signal_connect (G_OBJECT (widget), "color_set", G_CALLBACK (color_select), dialog);
+  widget = WID ("bg_colorbutton");
+  g_signal_connect (G_OBJECT (widget), "color_set", G_CALLBACK (color_select), dialog);
+  widget = WID ("text_colorbutton");
+  g_signal_connect (G_OBJECT (widget), "color_set", G_CALLBACK (color_select), dialog);
+  widget = WID ("base_colorbutton");
+  g_signal_connect (G_OBJECT (widget), "color_set", G_CALLBACK (color_select), dialog);
+  widget = WID ("selected_fg_colorbutton");
+  g_signal_connect (G_OBJECT (widget), "color_set", G_CALLBACK (color_select), dialog);
+  widget = WID ("selected_bg_colorbutton");
+  g_signal_connect (G_OBJECT (widget), "color_set", G_CALLBACK (color_select), dialog);
 
+  widget = WID ("enable_custom_colors_checkbutton");
+  g_signal_connect (G_OBJECT (widget), "toggled", G_CALLBACK (toggle_color_scheme_key), parent);
+
+  widget = WID ("save_color_scheme_button");
+  g_signal_connect (G_OBJECT (widget), "clicked", G_CALLBACK (save_color_scheme), parent);
+
+  widget = WID ("color_scheme_combobox");
+  g_signal_connect (G_OBJECT (widget), "changed", G_CALLBACK (color_scheme_combobox_changed), parent);
+
+  widget = WID ("delete_color_scheme_button");
+  g_signal_connect (G_OBJECT (widget), "clicked", G_CALLBACK (delete_color_scheme), parent);
+
+  g_signal_connect (G_OBJECT (WID ("theme_notebook")), "switch-page", G_CALLBACK (theme_notebook_changed_page), dialog);
+
+  g_object_get (G_OBJECT (gtk_settings_get_default()), "gtk-color-scheme", &color_scheme, NULL);
+  update_color_buttons_from_string (color_scheme);
+
+  /* general signals */
   g_signal_connect (G_OBJECT (parent), "response", G_CALLBACK (cb_dialog_response), NULL);
   g_signal_connect (G_OBJECT (parent), "delete-event", G_CALLBACK (gtk_true), NULL);
 
@@ -716,15 +957,86 @@ update_list_something (GtkWidget *tree_view, const gchar *theme)
     }
 }
 
+void
+update_color_buttons_from_string (gchar *color_scheme)
+{
+  GdkColor color_scheme_colors[6];
+  gchar **color_scheme_strings, **color_scheme_pair, *current_string;
+  gint i;
+  GtkWidget *widget;
+  GladeXML *dialog;
+
+  if (!color_scheme) return;
+  if (!strcmp (color_scheme, "")) return;
+
+  dialog = gnome_theme_manager_get_theme_dialog ();
+
+  /* The color scheme string consists of name:color pairs, seperated by
+   * newlines, so first we split the string up by new line */
+
+  color_scheme_strings = g_strsplit (color_scheme, "\n", 0);
+
+  /* loop through the name:color pairs, and save the colour if we recognise the name */
+  i = 0;
+  while ((current_string = color_scheme_strings[i++]))
+  {
+    color_scheme_pair = g_strsplit (current_string, ":", 0);
+
+    if (color_scheme_pair[0] == NULL)
+      continue;
+
+    g_strstrip (color_scheme_pair[0]);
+    g_strstrip (color_scheme_pair[1]);
+
+
+    if (!strcmp ("fg_color", color_scheme_pair[0]))
+      gdk_color_parse (color_scheme_pair[1], &color_scheme_colors[0]);
+    else if (!strcmp ("bg_color", color_scheme_pair[0]))
+      gdk_color_parse (color_scheme_pair[1], &color_scheme_colors[1]);
+    else if (!strcmp ("text_color", color_scheme_pair[0]))
+      gdk_color_parse (color_scheme_pair[1], &color_scheme_colors[2]);
+    else if (!strcmp ("base_color", color_scheme_pair[0]))
+      gdk_color_parse (color_scheme_pair[1], &color_scheme_colors[3]);
+    else if (!strcmp ("selected_fg_color", color_scheme_pair[0]))
+      gdk_color_parse (color_scheme_pair[1], &color_scheme_colors[4]);
+    else if (!strcmp ("selected_bg_color", color_scheme_pair[0]))
+      gdk_color_parse (color_scheme_pair[1], &color_scheme_colors[5]);
+
+    g_strfreev (color_scheme_pair);
+  }
+
+  g_strfreev (color_scheme_strings);
+
+  /* not sure whether we need to do this, but it can't hurt */
+  for (i = 0; i < 6; i++)
+    gdk_colormap_alloc_color (gdk_colormap_get_system (), &color_scheme_colors[i], FALSE, TRUE);
+
+  /* now set all the buttons to the correct settings */
+  widget = WID ("fg_colorbutton");
+  gtk_color_button_set_color (GTK_COLOR_BUTTON (widget), &color_scheme_colors[0]);
+  widget = WID ("bg_colorbutton");
+  gtk_color_button_set_color (GTK_COLOR_BUTTON (widget), &color_scheme_colors[1]);
+  widget = WID ("text_colorbutton");
+  gtk_color_button_set_color (GTK_COLOR_BUTTON (widget), &color_scheme_colors[2]);
+  widget = WID ("base_colorbutton");
+  gtk_color_button_set_color (GTK_COLOR_BUTTON (widget), &color_scheme_colors[3]);
+  widget = WID ("selected_fg_colorbutton");
+  gtk_color_button_set_color (GTK_COLOR_BUTTON (widget), &color_scheme_colors[4]);
+  widget = WID ("selected_bg_colorbutton");
+  gtk_color_button_set_color (GTK_COLOR_BUTTON (widget), &color_scheme_colors[5]);
+
+}
+
 void
 gnome_theme_details_update_from_gconf (void)
 {
   GConfClient *client;
   GladeXML *dialog;
-  GtkWidget *tree_view;
-  gchar *theme;
+  GtkWidget *tree_view, *widget;
+  gchar *theme = NULL;
   GnomeWindowManager *window_manager = NULL;
   GnomeWMSettings wm_settings;
+  gboolean use_custom_colors = FALSE;
 
   gnome_theme_details_init ();
 
@@ -749,6 +1061,28 @@ gnome_theme_details_update_from_gconf (void)
   tree_view = WID ("icon_theme_treeview");
   theme = gconf_client_get_string (client, ICON_THEME_KEY, NULL);
   update_list_something (tree_view, theme);
-  g_object_unref (client);
   g_free (theme);
+
+
+  /* color preferences */
+
+  theme = gconf_client_get_string (client, COLOR_SCHEME_KEY, NULL);
+  if (theme)
+  {
+    use_custom_colors = strcmp ("", theme);
+    update_color_buttons_from_string (theme);
+    g_free (theme);
+  }
+
+  widget = WID ("enable_custom_colors_checkbutton");
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), use_custom_colors);
+
+  widget = WID ("saved_color_schemes_hbox");
+  gtk_widget_set_sensitive (widget, use_custom_colors);
+
+  widget = WID ("color_scheme_table");
+  gtk_widget_set_sensitive (widget, use_custom_colors);
+
+
+  g_object_unref (client);
 }
