@@ -458,9 +458,16 @@ transfer_done_cb (GtkWidget *dlg, gchar *path)
 		if (theme_type == THEME_GTK || theme_type == THEME_METACITY || theme_type == THEME_ICON)
 		{
 			/* TODO: currently cannot apply "gnome themes" */
-			theme_props->user_message=g_strdup_printf(_("<span weight=\"bold\" size=\"larger\">The theme \"%s\" has been installed.</span>\n\nWould you like to apply it now, or keep your current theme?"), theme_props->theme_name);
+			gchar* str;
+
+			str = g_strdup_printf(_("The theme \"%s\" has been installed."), theme_props->theme_name);
+			theme_props->user_message=g_strdup_printf("<span weight=\"bold\" size=\"larger\">%s</span>", str);
+			g_free(str);
+
 			dialog = gtk_message_dialog_new_with_markup (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_INFO, GTK_BUTTONS_NONE, theme_props->user_message );
 
+			gtk_message_dialog_format_secondary_text(dialog, _("Would you like to apply it now, or keep your current theme?"));
+
 			gtk_dialog_add_button (GTK_DIALOG (dialog), _("Keep Current Theme"), GTK_RESPONSE_CLOSE);
 
 			apply_button = gtk_button_new_with_label (_("Apply New Theme"));
@@ -572,7 +579,7 @@ gnome_theme_install_from_uri (gchar * theme_filename, GtkWindow *parent)
 			GtkWidget *dialog;
 		  	int len = strlen (base);
 		  
-		  	if (base && len > 7 && ( (!strcmp (base + len - 7, ".tar.gz")) || (!strcmp (base + len - 4, ".tgz")) ))
+		  	if (base && len > 7 && ( (!strcmp (base + len - 7, ".tar.gz")) || (!strcmp (base + len - 4, ".tgz")) || (!strcmp (base + len - 4, ".gtp"))))
 		    		file_tmp = g_strdup_printf("gnome-theme-%d.tar.gz", rand ());
 		  	else if (base && len > 8 && !strcmp (base + len - 8, ".tar.bz2"))
 		    		file_tmp = g_strdup_printf("gnome-theme-%d.tar.bz2", rand ());
